@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2016 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2017 - pancake, nibble */
 
 #ifndef R2_PARSE_H
 #define R2_PARSE_H
@@ -22,9 +22,12 @@ typedef struct r_parse_t {
 	void *user;
 	int flagspace;
 	int notin_flagspace;
+	bool pseudo;
 	bool relsub; // replace rip relative expressions in instruction
 	bool localvar_only; // if true use only the local variable name (e.g. [local_10h] instead of [ebp + local10h])
 	int relsub_addr;
+	int minval;
+	char *retleave_asm;
 	struct r_parse_plugin_t *cur;
 	RAnal *anal; // weak anal ref
 	RAnalHint *hint; // weak anal ref
@@ -49,28 +52,32 @@ typedef struct r_parse_plugin_t {
 R_API struct r_parse_t *r_parse_new(void);
 R_API void r_parse_free(RParse *p);
 R_API void r_parse_set_user_ptr(RParse *p, void *user);
-R_API int r_parse_add(RParse *p, struct r_parse_plugin_t *foo);
+R_API int r_parse_add(RParse *p, RParsePlugin *foo);
 R_API int r_parse_list(RParse *p);
 R_API int r_parse_use(RParse *p, const char *name);
 R_API int r_parse_parse(RParse *p, const char *data, char *str);
 R_API int r_parse_assemble(RParse *p, char *data, char *str);
 R_API int r_parse_filter(RParse *p, RFlag *f, char *data, char *str, int len, bool big_endian);
 R_API bool r_parse_varsub(RParse *p, RAnalFunction *f, ut64 addr, int oplen, char *data, char *str, int len);
-R_API char *r_parse_c_string(const char *code);
-R_API char *r_parse_c_file(const char *path);
+R_API char *r_parse_c_string(RAnal *anal, const char *code);
+R_API char *r_parse_c_file(RAnal *anal, const char *path);
 R_API int r_parse_is_c_file (const char *file);
+R_API char *r_parse_immtrim (char *opstr);
 
 /* plugin pointers */
-extern struct r_parse_plugin_t r_parse_plugin_dummy;
-extern struct r_parse_plugin_t r_parse_plugin_att2intel;
-extern struct r_parse_plugin_t r_parse_plugin_x86_pseudo;
-extern struct r_parse_plugin_t r_parse_plugin_arm_pseudo;
-extern struct r_parse_plugin_t r_parse_plugin_mips_pseudo;
-extern struct r_parse_plugin_t r_parse_plugin_dalvik_pseudo;
-extern struct r_parse_plugin_t r_parse_plugin_mreplace;
-extern struct r_parse_plugin_t r_parse_plugin_ppc_pseudo;
-extern struct r_parse_plugin_t r_parse_plugin_6502_pseudo;
-extern struct r_parse_plugin_t r_parse_plugin_m68k_pseudo;
+extern RParsePlugin r_parse_plugin_dummy;
+extern RParsePlugin r_parse_plugin_att2intel;
+extern RParsePlugin r_parse_plugin_x86_pseudo;
+extern RParsePlugin r_parse_plugin_arm_pseudo;
+extern RParsePlugin r_parse_plugin_mips_pseudo;
+extern RParsePlugin r_parse_plugin_dalvik_pseudo;
+extern RParsePlugin r_parse_plugin_mreplace;
+extern RParsePlugin r_parse_plugin_ppc_pseudo;
+extern RParsePlugin r_parse_plugin_sh_pseudo;
+extern RParsePlugin r_parse_plugin_avr_pseudo;
+extern RParsePlugin r_parse_plugin_6502_pseudo;
+extern RParsePlugin r_parse_plugin_m68k_pseudo;
+extern RParsePlugin r_parse_plugin_z80_pseudo;
 #endif
 
 #ifdef __cplusplus
