@@ -12,7 +12,7 @@
 extern "C" {
 #endif
 
-R_LIB_VERSION_HEADER(r_socket);
+R_LIB_VERSION_HEADER (r_socket);
 
 #if __UNIX__ || __CYGWIN__ || __MINGW64__ && !defined(MINGW32)
 #include <netinet/in.h>
@@ -42,7 +42,6 @@ R_LIB_VERSION_HEADER(r_socket);
 #define SD_SEND 1
 #define SD_BOTH 2
 #endif
-
 typedef struct {
 	int magic;
 	int child;
@@ -61,7 +60,7 @@ typedef struct r_socket_t {
 	int fd;
 #endif
 	int is_ssl;
-	int local; // TODO: merge ssl with local -> flags/options
+	int local;	// TODO: merge ssl with local -> flags/options
 	int port;
 	struct sockaddr_in sa;
 #if HAVE_LIB_SSL
@@ -76,34 +75,36 @@ typedef struct r_socket_t {
 #define R_SOCKET_PROTO_UNIX 0x1337
 
 #ifdef R_API
-R_API RSocket *r_socket_new_from_fd (int fd);
-R_API RSocket *r_socket_new (int is_ssl);
-R_API bool r_socket_connect (RSocket *s, const char *host, const char *port, int proto, unsigned int timeout);
+R_API RSocket *r_socket_new_from_fd(int fd);
+R_API RSocket *r_socket_new(int is_ssl);
+R_API bool r_socket_connect(RSocket *s, const char *host, const char *port, int proto, unsigned int timeout);
+R_API bool r_socket_spawn (RSocket *s, const char *cmd, unsigned int timeout);
 R_API int r_socket_connect_serial(RSocket *sock, const char *path, int speed, int parity);
-#define r_socket_connect_tcp(a,b,c,d) r_socket_connect(a,b,c,R_SOCKET_PROTO_TCP,d)
-#define r_socket_connect_udp(a,b,c,d) r_socket_connect(a,b,c,R_SOCKET_PROTO_UDP,d)
+#define r_socket_connect_tcp(a, b, c, d) r_socket_connect (a, b, c, R_SOCKET_PROTO_TCP, d)
+#define r_socket_connect_udp(a, b, c, d) r_socket_connect (a, b, c, R_SOCKET_PROTO_UDP, d)
 #if __UNIX__
-#define r_socket_connect_unix(a,b) r_socket_connect(a,b,NULL,R_SOCKET_PROTO_UNIX)
-R_API int r_socket_unix_listen (RSocket *s, const char *file);
+#define r_socket_connect_unix(a, b) r_socket_connect (a, b, NULL, R_SOCKET_PROTO_UNIX)
+R_API int r_socket_unix_listen(RSocket *s, const char *file);
 #endif
 R_API int r_socket_port_by_name(const char *name);
-R_API int r_socket_close_fd (RSocket *s);
-R_API int r_socket_close (RSocket *s);
-R_API int r_socket_free (RSocket *s);
-R_API bool r_socket_listen (RSocket *s, const char *port, const char *certfile);
-R_API RSocket *r_socket_accept (RSocket *s);
-R_API int r_socket_block_time (RSocket *s, int block, int sec);
-R_API int r_socket_flush (RSocket *s);
-R_API int r_socket_ready (RSocket *s, int secs, int usecs);
-R_API char *r_socket_to_string (RSocket *s);
-R_API int r_socket_write (RSocket *s, void *buf, int len);
-R_API int r_socket_puts (RSocket *s, char *buf);
-R_API void r_socket_printf (RSocket *s, const char *fmt, ...);
-R_API int r_socket_read (RSocket *s, ut8 *read, int len);
-R_API int r_socket_read_block (RSocket *s, unsigned char *buf, int len);
-R_API int r_socket_gets (RSocket *s, char *buf, int size);
-R_API ut8* r_socket_slurp(RSocket *s, int *len);
-R_API bool r_socket_is_connected (RSocket *);
+R_API int r_socket_close_fd(RSocket *s);
+R_API int r_socket_close(RSocket *s);
+R_API int r_socket_free(RSocket *s);
+R_API bool r_socket_listen(RSocket *s, const char *port, const char *certfile);
+R_API RSocket *r_socket_accept(RSocket *s);
+R_API RSocket *r_socket_accept_timeout(RSocket *s, unsigned int timeout);
+R_API int r_socket_block_time(RSocket *s, int block, int sec);
+R_API int r_socket_flush(RSocket *s);
+R_API int r_socket_ready(RSocket *s, int secs, int usecs);
+R_API char *r_socket_to_string(RSocket *s);
+R_API int r_socket_write(RSocket *s, void *buf, int len);
+R_API int r_socket_puts(RSocket *s, char *buf);
+R_API void r_socket_printf(RSocket *s, const char *fmt, ...);
+R_API int r_socket_read(RSocket *s, ut8 *read, int len);
+R_API int r_socket_read_block(RSocket *s, unsigned char *buf, int len);
+R_API int r_socket_gets(RSocket *s, char *buf, int size);
+R_API ut8 *r_socket_slurp(RSocket *s, int *len);
+R_API bool r_socket_is_connected(RSocket *);
 
 /* process */
 typedef struct r_socket_proc_t {
@@ -112,17 +113,17 @@ typedef struct r_socket_proc_t {
 	int pid;
 } RSocketProc;
 
-R_API RSocketProc *r_socket_proc_open(char* const argv[]);
+R_API RSocketProc *r_socket_proc_open(char *const argv[]);
 R_API int r_socket_proc_close(RSocketProc *sp);
-R_API int r_socket_proc_read (RSocketProc *sp, unsigned char *buf, int len);
-R_API int r_socket_proc_gets (RSocketProc *sp, char *buf, int size);
-R_API int r_socket_proc_write (RSocketProc *sp, void *buf, int len);
-R_API void r_socket_proc_printf (RSocketProc *sp, const char *fmt, ...);
-R_API int r_socket_proc_ready (RSocketProc *sp, int secs, int usecs);
+R_API int r_socket_proc_read(RSocketProc *sp, unsigned char *buf, int len);
+R_API int r_socket_proc_gets(RSocketProc *sp, char *buf, int size);
+R_API int r_socket_proc_write(RSocketProc *sp, void *buf, int len);
+R_API void r_socket_proc_printf(RSocketProc *sp, const char *fmt, ...);
+R_API int r_socket_proc_ready(RSocketProc *sp, int secs, int usecs);
 
 /* HTTP */
-R_API char *r_socket_http_get (const char *url, int *code, int *rlen);
-R_API char *r_socket_http_post (const char *url, const char *data, int *code, int *rlen);
+R_API char *r_socket_http_get(const char *url, int *code, int *rlen);
+R_API char *r_socket_http_post(const char *url, const char *data, int *code, int *rlen);
 R_API void r_socket_http_server_set_breaked(bool *b);
 
 typedef struct r_socket_http_request {
@@ -136,9 +137,9 @@ typedef struct r_socket_http_request {
 	int data_length;
 } RSocketHTTPRequest;
 
-R_API RSocketHTTPRequest *r_socket_http_accept (RSocket *s, int timeout);
-R_API void r_socket_http_response (RSocketHTTPRequest *rs, int code, const char *out, int x, const char *headers);
-R_API void r_socket_http_close (RSocketHTTPRequest *rs);
+R_API RSocketHTTPRequest *r_socket_http_accept(RSocket *s, int accept_timeout, int timeout);
+R_API void r_socket_http_response(RSocketHTTPRequest *rs, int code, const char *out, int x, const char *headers);
+R_API void r_socket_http_close(RSocketHTTPRequest *rs);
 R_API ut8 *r_socket_http_handle_upload(const ut8 *str, int len, int *olen);
 
 typedef int (*rap_server_open)(void *user, const char *file, int flg, int mode);
@@ -154,7 +155,7 @@ enum {
 	RAP_RMT_WRITE,
 	RAP_RMT_SEEK,
 	RAP_RMT_CLOSE,
-	RAP_RMT_CMD,
+	RAP_RMT_CMD = 0x07,
 	RAP_RMT_REPLY = 0x80,
 	RAP_RMT_MAX = 4096
 };
@@ -162,7 +163,7 @@ enum {
 typedef struct r_socket_rap_server_t {
 	RSocket *fd;
 	char port[5];
-	ut8 buf[RAP_RMT_MAX + 32]; //This should be used as a static buffer for everything done by the server
+	ut8 buf[RAP_RMT_MAX + 32];	// This should be used as a static buffer for everything done by the server
 	rap_server_open open;
 	rap_server_seek seek;
 	rap_server_read read;
@@ -170,22 +171,25 @@ typedef struct r_socket_rap_server_t {
 	rap_server_cmd system;
 	rap_server_cmd cmd;
 	rap_server_close close;
-	void *user; //Always first arg for callbacks
+	void *user;	// Always first arg for callbacks
 } RSocketRapServer;
 
-R_API RSocketRapServer *r_socket_rap_server_new (int is_ssl, const char *port);
-R_API RSocketRapServer *r_socket_rap_server_create (const char *pathname);
-R_API void r_socket_rap_server_free (RSocketRapServer *rap_s);
-R_API int r_socket_rap_server_listen (RSocketRapServer *rap_s, const char *certfile);
-R_API RSocket* r_socket_rap_server_accept (RSocketRapServer *rap_s);
-R_API bool r_socket_rap_server_continue (RSocketRapServer *rap_s);
+R_API RSocketRapServer *r_socket_rap_server_new(int is_ssl, const char *port);
+R_API RSocketRapServer *r_socket_rap_server_create(const char *pathname);
+R_API void r_socket_rap_server_free(RSocketRapServer *rap_s);
+R_API int r_socket_rap_server_listen(RSocketRapServer *rap_s, const char *certfile);
+R_API RSocket *r_socket_rap_server_accept(RSocketRapServer *rap_s);
+R_API bool r_socket_rap_server_continue(RSocketRapServer *rap_s);
 
 /* run.c */
 #define R_RUN_PROFILE_NARGS 512
 typedef struct r_run_profile_t {
 	char *_args[R_RUN_PROFILE_NARGS];
+	int _argc;
 	char *_system;
 	char *_program;
+	char *_runlib;
+	char *_runlib_fcn;
 	char *_stdio;
 	char *_stdin;
 	char *_stdout;
@@ -222,20 +226,20 @@ typedef struct r_run_profile_t {
 
 R_API RRunProfile *r_run_new(const char *str);
 R_API bool r_run_parse(RRunProfile *pf, const char *profile);
-R_API void r_run_free (RRunProfile *r);
-R_API bool r_run_parseline (RRunProfile *p, char *b);
+R_API void r_run_free(RRunProfile *r);
+R_API bool r_run_parseline(RRunProfile *p, char *b);
 R_API const char *r_run_help(void);
 R_API int r_run_config_env(RRunProfile *p);
 R_API int r_run_start(RRunProfile *p);
 R_API void r_run_reset(RRunProfile *p);
-R_API int r_run_parsefile (RRunProfile *p, const char *b);
+R_API int r_run_parsefile(RRunProfile *p, const char *b);
 
 /* r2pipe */
 R_API int r2p_close(R2Pipe *r2p);
 R_API R2Pipe *r2p_open(const char *cmd);
 R_API int r2p_write(R2Pipe *r2p, const char *str);
 R_API char *r2p_read(R2Pipe *r2p);
-R_API void r2p_free (R2Pipe *r2p);
+R_API void r2p_free(R2Pipe *r2p);
 R_API char *r2p_cmd(R2Pipe *r2p, const char *str);
 R_API char *r2p_cmdf(R2Pipe *r2p, const char *fmt, ...);
 #endif
