@@ -4,6 +4,7 @@
 // defines like IS_DIGIT, etc'
 #include "r_util/r_str_util.h"
 #include "r_userconf.h"
+#include <stddef.h>
 
 // TODO: fix this to make it crosscompile-friendly: R_SYS_OSTYPE ?
 /* operating system */
@@ -98,7 +99,7 @@
   #define __BSD__ 0
   #define __UNIX__ 1
 #endif
-#if __KFBSD__ || defined(__NetBSD__) || defined(__OpenBSD__)
+#if __KFBSD__ || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__)
   #define __BSD__ 1
   #define __UNIX__ 1
 #endif
@@ -256,7 +257,7 @@ R_API const char *x##_version () { return "" R2_GITTAP; }
 #define BITS2BYTES(x) (((x)/8)+(((x)%8)?1:0))
 #define ZERO_FILL(x) memset (&x, 0, sizeof (x))
 #define R_NEWS0(x,y) (x*)calloc(y,sizeof(x))
-#define R_NEWS(x,y) (x*)malloc(sizeof(x)*y)
+#define R_NEWS(x,y) (x*)malloc(sizeof(x)*(y))
 #define R_NEW0(x) (x*)calloc(1,sizeof(x))
 #define R_NEW(x) (x*)malloc(sizeof(x))
 #define R_NEWCOPY(x,y) (x*)r_new_copy(sizeof(x), y)
@@ -330,7 +331,7 @@ static inline void *r_new_copy(int size, void *data) {
 #endif
 
 #undef r_offsetof
-#define r_offsetof(type, member) ((unsigned long) &((type*)0)->member)
+#define r_offsetof(type, member) offsetof(type, member)
 
 #define R_BETWEEN(x,y,z) (((y)>=(x)) && ((y)<=(z)))
 #define R_ROUND(x,y) ((x)%(y))?(x)+((y)-((x)%(y))):(x)
